/*
 * adc_spi.c
 *
 *  Created on: Jan 15, 2012
 *      Author: Kurt Snieckus
 *
 *      Functions to configure, initiate and read back conversions from the ADC
 */

#include <msp430.h>
#include <msp430x26x.h>
#include "adc_spi.h"

unsigned long adc_reading(char chan, char nextchan, int avg) {
	// Reads "avg" samples from adc channel "chan", assuming you told adc
	// to measure for channel "chan" twice before running this.
	// The last two measurements the configuration sent is

	unsigned long long reading = 0;
	int i;

	for (i = 0; i < avg+3; i++) {

		if (i == 3) reading = 0; // Reset readings because it has values from previous configs

		// Configure for INk - Convert INk-1 - Return INk-2
		// ADC Channel config
		// CFG (BIT15) = 1: overwrite register
		// INCC (BIT14:BIT12) = 110: Bipolar single ended
		// INx (BIT11:BIT9) = 001: IN1 selected
		// BW (BIT8) = 1: full bw
		// REF (BIT7:BIT5) = 001: 4.096 internal ref
		// SEQ (BIT4:BIT3) = 00: Channel Sequencer off
		// RB (BIT2) = 1: do not read back config
		reading += adc_convert(0x8000 + 0x6000 + ((chan & 0x07) << 9) + BIT8 + BIT5 + BIT2);
	}


	return (long) (reading/avg); // Divide reading by the average
}

unsigned long adc_convert(int next_config) {
	// Initiate conversion, return result from previous conversion
	// and configure next conversion
	// The first few times this function is run it will potentially
	// produce partially garbage outputs.
	// next_config is the MSB justified configuration for the next conversion
	unsigned long output;

	// Hold CNV (P2.0) High for more than 2.2uS
	P3OUT |= BIT0;
	__delay_cycles(40); // delay by 40 cycles (~2.5uS)
	P3OUT &= ~BIT0;

	// Bit of a hack because the ADC doesn't play nice with SPI
	// The ADC sets the MSB on the SDO as soon as CNV returns low.
	// The MCU doesn't read this first bit and instead starts reading at the next byte, missing the MSB
	// So this retrieves the MSB before starting the hardware SPI comm
	__delay_cycles(5); // Wait a bit for the ADC to set it
	P3SEL &= ~BIT2;		// Set MISO to GPIO
	output = (((int)P3IN) << 13) & 0x8000; // Read value
	P3SEL |= BIT2;		// Return MISO to hardware USART



	adc_spi_tx(next_config >> 8); 	// transfer high byte  of configuration for next conversion

	output |= ((int)adc_spi_rx()) << 7;		// read high byte of previous conversion
	// only move it up 7 because the MSB is up at the top

	adc_spi_tx(0xFF & next_config);			// transfer low byte of configuration for next conversion

	output |= (adc_spi_rx() >> 1);   // read low byte of previous conversion

	return output;

}

void adc_spi_init(void) {
	// Setup SPI communication with ADC
	// On PCB R01 -> UCB0

	// Set UCSWRST (BIS.B #UCSWRST,&UCxCTL1).
	// Initialize all eUSCI registers with UCSWRST = 1 (including UCxCTL1).
	// Configure ports.
	// Clear UCSWRST via software (BIC.B #UCSWRST,&UCxCTL1).
	// Enable interrupts (optional) via UCRXIE and/or UCTXIE.

	//UCB0CTL1 |= UCSWRST;	// Hold USCI in reset

	// UCB0CTL0:
	// ~UCCKPH: Data is written on the falling edge, and captured by the ADC on the rising edge.
	// UCCKPL: CLK is kept high when not in use.
	// UCMSB: MSB first
	// ~UC7BIT: 8-bit data mode
	// UCMST: Master Mode
	// UCMODE_0: 3-pin SPI
	// UCSYNC: Synchronous mode (for SPI)

	UCB0CTL0 |= UCCKPL+UCMSB+UCMST+UCSYNC;    //3-pin, 8-bit SPI master

	UCB0CTL1 |= UCSSEL_2;          // SMCLK

	UCB0BR0 = 0xFF;                           //
	UCB0BR1 = 0;                              //

	UCB0CTL1 &= ~UCSWRST;                     // **Initialize USCI state machine**

	// Setup DIN/SIMO (P3.1) SDO/SOMI (P3.2) SCK/CLK (3.3)
	P3SEL |= 0x0E;
	P3DIR &= ~BIT2;		// Set SOMI as input

	// Setup CNV (P3.0) as output
	P3DIR |= BIT0;
	P3SEL &= ~BIT0;

}

void adc_spi_tx(char data) {
	// transmit byte to ADC over SPI

	// Wait for TX buffer to be available
	while (!(IFG2 & UCB0TXIFG));
	UCB0TXBUF = data;
}

char adc_spi_rx(void) {
	// Receive byte from ADC

	while (!(IFG2 & UCB0RXIFG));
	return (unsigned char)UCB0RXBUF;
}




